package sun.net.www;

import java.util.Iterator;

public class HeaderParser
{
    String raw;
    String[][] tab;
    int nkeys;
    int asize;
    
    public HeaderParser(String raw) {
        this.asize = 10;
        this.raw = raw;
        this.tab = new String[this.asize][2];
        this.parse();
    }
    
    private HeaderParser() {
        this.asize = 10;
    }
    
    public HeaderParser subsequence(int n, int n2) {
        if (n == 0 && n2 == this.nkeys) {
            return this;
        }
        if (n < 0 || n >= n2 || n2 > this.nkeys) {
            throw new IllegalArgumentException("invalid start or end");
        }
        HeaderParser headerParser = new HeaderParser();
        headerParser.tab = new String[this.asize][2];
        headerParser.asize = this.asize;
        System.arraycopy(this.tab, n, headerParser.tab, 0, n2 - n);
        headerParser.nkeys = n2 - n;
        return headerParser;
    }
    
    private void parse() {
        if (this.raw != null) {
            this.raw = this.raw.trim();
            char[] charArray = this.raw.toCharArray();
            int offset = 0;
            int i = 0;
            int nkeys = 0;
            int n = 1;
            int n2 = 0;
            int length = charArray.length;
            while (i < length) {
                int n3 = charArray[i];
                if (n3 == '=' && n2 == 0) {
                    this.tab[nkeys][0] = new String(charArray, offset, i - offset).toLowerCase();
                    n = 0;
                    offset = ++i;
                }
                else if (n3 == '\"') {
                    if (n2 != 0) {
                        this.tab[nkeys++][1] = new String(charArray, offset, i - offset);
                        n2 = 0;
                        while (++i < length && (charArray[i] == ' ' || charArray[i] == ',')) {}
                        n = 1;
                        offset = i;
                    }
                    else {
                        n2 = 1;
                        offset = ++i;
                    }
                }
                else if (n3 == ' ' || n3 == ',') {
                    if (n2 != 0) {
                        ++i;
                        continue;
                    }
                    if (n != 0) {
                        this.tab[nkeys++][0] = new String(charArray, offset, i - offset).toLowerCase();
                    }
                    else {
                        this.tab[nkeys++][1] = new String(charArray, offset, i - offset);
                    }
                    while (i < length && (charArray[i] == ' ' || charArray[i] == ',')) {
                        ++i;
                    }
                    n = 1;
                    offset = i;
                }
                else {
                    ++i;
                }
                if (nkeys == this.asize) {
                    this.asize *= 2;
                    String[][] tab = new String[this.asize][2];
                    System.arraycopy(this.tab, 0, tab, 0, this.tab.length);
                    this.tab = tab;
                }
                else {
                    continue;
                }
            }
            if (--i > offset) {
                if (n == 0) {
                    if (charArray[i] == '\"') {
                        this.tab[nkeys++][1] = new String(charArray, offset, i - offset);
                    }
                    else {
                        this.tab[nkeys++][1] = new String(charArray, offset, i - offset + 1);
                    }
                }
                else {
                    this.tab[nkeys++][0] = new String(charArray, offset, i - offset + 1).toLowerCase();
                }
            }
            else if (i == offset) {
                if (n == 0) {
                    if (charArray[i] == '\"') {
                        this.tab[nkeys++][1] = String.valueOf(charArray[i - 1]);
                    }
                    else {
                        this.tab[nkeys++][1] = String.valueOf(charArray[i]);
                    }
                }
                else {
                    this.tab[nkeys++][0] = String.valueOf(charArray[i]).toLowerCase();
                }
            }
            this.nkeys = nkeys;
        }
    }
    
    public String findKey(int n) {
        if (n < 0 || n > this.asize) {
            return null;
        }
        return this.tab[n][0];
    }
    
    public String findValue(int n) {
        if (n < 0 || n > this.asize) {
            return null;
        }
        return this.tab[n][1];
    }
    
    public String findValue(String s) {
        return this.findValue(s, null);
    }
    
    public String findValue(String lowerCase, String s) {
        if (lowerCase == null) {
            return s;
        }
        lowerCase = lowerCase.toLowerCase();
        int i = 0;
        while (i < this.asize) {
            if (this.tab[i][0] == null) {
                return s;
            }
            if (lowerCase.equals(this.tab[i][0])) {
                return this.tab[i][1];
            }
            ++i;
        }
        return s;
    }
    
    public Iterator<String> keys() {
        return new HeaderParser.ParserIterator(false);
    }
    
    public Iterator<String> values() {
        return new HeaderParser.ParserIterator(true);
    }
    
    @Override
    public String toString() {
        Iterator<String> keys = this.keys();
        StringBuffer buffer = new StringBuffer();
        buffer.append("{size=" + this.asize + " nkeys=" + this.nkeys + " ");
        for (int n = 0; keys.hasNext(); ++n) {
            String str = (String)keys.next();
            String value = this.findValue(n);
            if (value != null && "".equals(value)) {
                value = null;
            }
            buffer.append(" {" + str + ((value == null) ? "" : ("," + value)) + "}");
            if (keys.hasNext()) {
                buffer.append(",");
            }
        }
        buffer.append(" }");
        return new String(buffer);
    }
    
    public int findInt(String s, int i) {
        try {
            return Integer.parseInt(this.findValue(s, String.valueOf(i)));
        }
        catch (Throwable t) {
            return i;
        }
    }
    
    class ParserIterator implements Iterator<String>
    {
        int index;
        boolean returnsValue;
        
        ParserIterator(boolean returnsValue) {
            this.returnsValue = returnsValue;
        }
        
        @Override
        public boolean hasNext() {
            return this.index < HeaderParser.this.nkeys;
        }
        
        @Override
        public String next() {
            return HeaderParser.this.tab[this.index++][this.returnsValue ? 1 : 0];
        }
        
        @Override
        public void remove() {
            throw new UnsupportedOperationException("remove not supported");
        }
    }
}
